! gfortran -fopenmp stream.f90 -o stream

module functions
use omp_lib
contains

function checktick(M)

integer :: checktick
integer, intent(in) :: M
integer :: i, mindelta, delta
double precision :: t1, t2
double precision, dimension(M) :: timesfound
! Collect a sequence of M unique time values from the system.
do i=1,M
	t1 = omp_get_wtime()
	do
		t2 = omp_get_wtime() - t1
		if ( t2 >= 1./1000000.) exit
		t1 = t2
		timesfound(i) = t1  
	end do 
end do
 ! Determine the minimum difference between these M values.
 ! This result will be our estimate (in microseconds) for the
 ! clock granularity.  
minDelta = 1000000
do i=1, M
	Delta = aint( 1000000. * (timesfound(i)-timesfound(i-1)))
	minDelta = min(mindelta, max(Delta,0))
end do
 checktick = mindelta	
end function checktick

end module functions

program main
use functions
use omp_lib
implicit none    
integer :: quantum
integer :: BytesPerWord
integer :: ntimes, offset, N, M, j, k
double precision ::	scalar, t
double precision, dimension(:), allocatable :: times
double precision, allocatable, dimension(:) :: a, b, c
double precision :: rmstime, maxtime, mintime
!double precision, dimension(:,:), allocatable :: bytes
!double precision, parameter :: bytes = 3 * 8 * N 
double precision :: bytes 
character(len=11) :: label = 'SAXPYing  :'
offset = 0 
ntimes = 10 
n = 10000000
bytes = 3 * 8 * n
!allocate (bytes(3,N), times(ntimes))
allocate (times(ntimes))

rmstime = 0.
maxtime = 0.
mintime = 3.402823466e+38

M = 20

    ! /* --- SETUP --- determine precision and check timing --- */

	write (*,*) '-------------------------------------------------------------'

    BytesPerWord = 8
    write(*,*) 'This system uses 8 bytes per DOUBLE PRECISION word.'
 
	write (*,*) '-------------------------------------------------------------'
	
	write (*,*) 'Array size = ', n, ' , Offset = ', offset
	write (*,*) 'Total memory required = ', 24*N/1048576.0 , 'MB.'
	write (*,*) 'Each test is run ', ntimes, ' , but only the *best* time for each is used.'
 
    !/* allocate dynamic arrays */
    allocate (a(n+offset), b(n+offset), c(n+offset))

    !/* Get initial value for system clock. */

	a = 1.0
	b = 2.0
	c = 0.0
  
	write (*,*) '-------------------------------------------------------------'

    if  ( checktick(20) >= 1 ) then !!! if  ( (quantum = checktick) >= 1)   
		write(*,*) 'Your clock granularity/precision appears to be', quantum, ' microseconds.'
    else
		write(*,*)'Your clock granularity appears to be less than one microsecond.'
	end if
	
	
    t = omp_get_wtime()
    
    do j=1, n-1
		a(j) = 2.0 * a(j)
	end do
	
    t = 1000000. * (omp_get_wtime() - t)

	write (*,*)'Each test below will take on the order of', aint(t), 'microseconds.'
    write (*,*) aint(t/quantum), ' clock ticks'
    
    write (*,*)'Increase the size of the arrays if this shows that'
    write (*,*)'you are not getting at least 20 clock ticks per test.'

	write (*,*) '-------------------------------------------------------------'

    write (*,*)'WARNING: The above is only a rough guideline.'
    write (*,*)'For best results, please be sure you know the'
    write (*,*)'precision of your system timer.'
    
	write (*,*) '-------------------------------------------------------------'
    
    !/*	--- MAIN LOOP --- repeat test cases NTIMES times --- */

    scalar = 3.0

!$omp parallel shared(times,c,a,b,scalar) private(k)
								!// CT
    do k=1, NTIMES

!$omp master
	times(k) = omp_get_wtime()
!$omp end master

!$omp barrier

!$omp do
	do j=1,n
	    a(j) = b(j) + scalar * c(j)
	end do

!$omp master
	times(k) = omp_get_wtime() - times(k)
!$omp end master

!$omp barrier

	end do
!$omp end parallel
    
   
    !/* free dynamic arrays */
 	deallocate(a,b,c)

    do k=1, NTIMES;
	    rmstime = rmstime + (times(k) * times(k))
	    mintime = min(mintime, times(k))
	    maxtime = max(maxtime, times(k))
	end do
    
    write (*,*) 'Function      Rate (MB/s)   RMS time     Min time     Max time'
	rmstime = sqrt(rmstime/ntimes)
	write (*,*) label, 0.000001 * bytes/mintime, rmstime, mintime, maxtime
	

end program main
